//Code for master device (ESP32/Arduino) + BLE

#include <BLEDevice.h>
#include <BLEServer.h>
#include <BLEUtils.h>
#include <BLE2902.h>
#include <Wire.h>
#include "BMP085.h"


BLEServer *pServer = NULL;
BLECharacteristic * pTxCharacteristic;
bool deviceConnected = false;
bool oldDeviceConnected = false;
float temperature; //this is for the barometer, it calls the temperature function first to get the correct value of air pressure
float txValue = 0;
String sensorName = "Unknown";
byte sensorAdd;

#define pin 33 //battery status pin
#define pin_green 23 // green LED
#define pin_red 19 // red LED
#define gnd 25 // virtual ground
//#define vcc 21 // virtual vcc

float pinValue;
float pinVoltage;

// See the following for generating UUIDs:
// https://www.uuidgenerator.net/

#define SERVICE_UUID           "6e400001-b5a3-f393-e0a9-e50e24dcca9e" // UART service UUID
#define CHARACTERISTIC_UUID_RX "6e400003-b5a3-f393-e0a9-e50e24dcca9e"
#define CHARACTERISTIC_UUID_TX "6e400002-b5a3-f393-e0a9-e50e24dcca9e"


class MyServerCallbacks: public BLEServerCallbacks {
    void onConnect(BLEServer* pServer) {
      deviceConnected = true;
    };

    void onDisconnect(BLEServer* pServer) {
      deviceConnected = false;
    }
};

class MyCallbacks: public BLECharacteristicCallbacks {
    void onWrite(BLECharacteristic *pCharacteristic) {
      std::string rxValue = pCharacteristic->getValue();

      if (rxValue.length() > 0) {
        Serial.println("*********");
        Serial.print("Received Value: ");
        for (int i = 0; i < rxValue.length(); i++)
          Serial.print(rxValue[i]);

        Serial.println();
        Serial.println("*********");
      }
    }
};

BMP085 myBarometer;

void setup() {
  Wire.begin();
  Serial.begin(115200);
  //myBarometer.init();
  
  pinMode(pin_green, OUTPUT);
  pinMode(pin_red, OUTPUT);

  pinMode(gnd, OUTPUT);
  digitalWrite(gnd, LOW); //force pin 25 to be ground

  /*pinMode(vcc, OUTPUT);
  digitalWrite(vcc, HIGH); //force pin 21 to be vcc*/

  // Create the BLE Device
  BLEDevice::init("Arduino ESP32 M5");

  // Create the BLE Server
  pServer = BLEDevice::createServer();
  pServer->setCallbacks(new MyServerCallbacks());

  // Create the BLE Service
  BLEService *pService = pServer->createService(SERVICE_UUID);

  // Create a BLE Characteristic
  pTxCharacteristic = pService->createCharacteristic(
                    CHARACTERISTIC_UUID_TX,
                    BLECharacteristic::PROPERTY_NOTIFY
                  );
                      
  pTxCharacteristic->addDescriptor(new BLE2902());

  BLECharacteristic * pRxCharacteristic = pService->createCharacteristic(
                       CHARACTERISTIC_UUID_RX,
                      BLECharacteristic::PROPERTY_WRITE
                    );

  pRxCharacteristic->setCallbacks(new MyCallbacks());

  // Start the service
  pService->start();

  // Start advertising
  pServer->getAdvertising()->start();
  /*Serial.println("Starting sensor...");
  delay(3000);
  Serial.println("Warming up...");
  delay(3000); //warming up for 3 seconds*/
  Serial.println("Waiting a client connection to notify...");
}

void loop() {

//--------------------------------------------------------------------------------------------------------------------------------------------------
  //battery monitor using LEDs
 /* pinValue = analogRead(pin); //raw digital values
  pinVoltage  = (pinValue/4095)*3.3; //voltage in V

  if (pinVoltage >= 3.1)
  {
    digitalWrite(pin_green, HIGH);   // turn on the green LED
    digitalWrite(pin_red, LOW);    // turn off the red LED
    Serial.print("voltage : "); Serial.println(pinVoltage);
  }

  else if (pinVoltage < 3.1)
  {
    digitalWrite(pin_red, HIGH);   // turn on the red LED
    digitalWrite(pin_green, LOW);    // turn off the green LED
    Serial.print("voltage : "); Serial.println(pinVoltage);
  }*/
  //-------------------------------------------------------------------------------------------------------------------------------------------------

  // 

  String str = sensorName + " : " + String(txValue); // the string to send via BLE

    if (deviceConnected) {
        uint8_t arr[30];
        for(int i = 0; i < str.length(); i++)
        {
         arr[i] = str[i];
        }
        Serial.println(str);
        //pTxCharacteristic->setValue(&txValue, 1);
        pTxCharacteristic->setValue(arr, str.length());
        pTxCharacteristic->notify();
        //txValue++;

//-----------------------------------------------------------------------------------------------------------------------------------
//scan I2C device and get data
        uint8_t error, address;

  Serial.println("Scanning for I2C address ...");
  for(address = 1; address < 127; address++)
  {
    Wire.beginTransmission(address);
    error = Wire.endTransmission();
    if (error == 0)
    {
      Serial.printf("I2C device found at address %d\n", address);

      if (address == 8)
      {
        sensorName = "Temperature";
        sensorAdd = address;
        txValue = RequestingTemperature(sensorAdd); //getting temperature from custom I2C sensor in °C
        break;
      }

      else if (address == 119)
      {
        myBarometer.init();
        
        sensorName = "Pressure";
        
        temperature = myBarometer.bmp085GetTemperature(myBarometer.bmp085ReadUT());
  
        txValue = (float)(myBarometer.bmp085GetPressure(myBarometer.bmp085ReadUP()))/100; //getting pressure values from the barometer, txValue is in hPa
        
        break;
      }

    }
  }

//-----------------------------------------------------------------------------------------------------------------------------------
        
        delay(2000); // bluetooth stack will go into congestion, if too many packets are sent
        
  }

    // disconnecting
    if (!deviceConnected && oldDeviceConnected) {
        delay(500); // give the bluetooth stack the chance to get things ready
        pServer->startAdvertising(); // restart advertising
        Serial.println("start advertising");
        oldDeviceConnected = deviceConnected;
    }
    // connecting
    if (deviceConnected && !oldDeviceConnected) {
    // do stuff here on connecting
        oldDeviceConnected = deviceConnected;
    }
}


//-------------------------------------------------------------------------------------------------------------------------------------------------------------------
//custom functions
float RequestingTemperature(byte sAdd)
{
  Wire.requestFrom(sAdd, 2);
  //The wire can only transfer 1 byte at a time.
  //We need to receive temperature values in 2 bytes since temperature is a float, but the values are received as integers.
  //If the temperature is 25.42 °C, we receive a byte with 25, and another byte with 42.
  //Once we have received all the data we add 25 and 42/100 to get back our float number.

  int data[2];
  float temperature;

  if (Wire.available()) 
  {
    data[0] = Wire.read();
    data[1] = Wire.read();
  }
  
   temperature = data[0] + (float)data[1]/100;

   return temperature;
}
